<?php
/**
 * IDEALIAGroup srl
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@idealiagroup.com so we can send you a copy immediately.
 *
 * @category   MSP
 * @package    MSP_CashOnDelivery
 * @copyright  Copyright (c) 2016 IDEALIAGroup srl (http://www.idealiagroup.com)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

namespace MSP\CashOnDelivery\Model\Total\Creditmemo;

use Magento\Sales\Model\Order\Creditmemo;
use MSP\CashOnDelivery\Helper\Data;
use MSP\CashOnDelivery\Api\CashondeliveryInterface;
use Magento\Framework\Pricing\PriceCurrencyInterface;

class Cashondelivery extends AbstractTotal
{

    /**
     * @var Data \MSP\CashOnDelivery\Helper\Data
     */
    protected $dataHelper;
    /**
     * @var Data \MSP\CashOnDelivery\Api\CashondeliveryInterface
     */
    protected $cashOnDeliveryInterface;
    /**
     * @var Data \Magento\Framework\Pricing\PriceCurrencyInterface
     */
    protected $priceCurrencyInterface;

    public function __construct(
        Data $dataHelper,
        CashondeliveryInterface $cashOnDeliveryInterface,
        PriceCurrencyInterface $priceCurrencyInterface,
        array $data = []
    )
    {
        parent::__construct($data);

        $this->dataHelper = $dataHelper;
        $this->cashOnDeliveryInterface = $cashOnDeliveryInterface;
        $this->priceCurrencyInterface = $priceCurrencyInterface;
    }

    public function collect(Creditmemo $creditmemo)
    {
        $order = $creditmemo->getOrder();

        $baseMspCodAmount = $creditmemo->getBaseMspCodAmount();
        if ($creditmemo->getHasChangedBaseMspCodAmount()) {
            if(($baseMspCodAmount > $order->getBaseMspCodAmount()) || ($baseMspCodAmount < 0)) {
                throw new \Magento\Framework\Exception\LocalizedException(
                    __('Maximum Cash on Delivery amount allowed to refund is: %1. Minimum is 0', $order->getBaseMspCodAmount())
                );
            }

            $country = $creditmemo->getShippingAddress()->getCountryId();
            $region = $creditmemo->getShippingAddress()->getRegionId();
            if (!$region && is_string($creditmemo->getShippingAddress()->getRegion())) {
                $region = $creditmemo->getShippingAddress()->getRegion();
            }

            $newMspCodBaseTaxAmount =  $creditmemo->getBaseMspCodTaxAmount();
            $newMspCodTaxAmount =  $creditmemo->getMspCodTaxAmount();
            $creditMemoBaseTaxAmount = $creditmemo->getBaseTaxAmount();
            $creditMemoTaxAmount = $creditmemo->getTaxAmount();
            $oldMspCodTaxAmount = $creditmemo->getBaseMspCodTaxAmount();
            $baseTaxAmount = $this->cashOnDeliveryInterface->getBaseTaxAmount($baseMspCodAmount);
            $taxAmount = $this->priceCurrencyInterface->convert($baseTaxAmount);

            if($oldMspCodTaxAmount < $baseTaxAmount) {
                $newMspCodBaseTaxAmount = $oldMspCodTaxAmount - $baseTaxAmount;
                $newMspCodTaxAmount = $oldMspCodTaxAmount - $taxAmount;
                $creditMemoBaseTaxAmount = $creditmemo->getBaseTaxAmount() - $newMspCodBaseTaxAmount;
                $creditMemoTaxAmount = $creditmemo->getTaxAmount() - $newMspCodTaxAmount;
                $creditMemoBaseGrandTotal = $creditmemo->getBaseGrandTotal() + $creditmemo->getBaseMspCodAmount() - $newMspCodBaseTaxAmount;
                $creditMemoGrandTotal = $creditmemo->getGrandTotal() + $creditmemo->getMspCodAmount() - $newMspCodTaxAmount;
            } elseif($oldMspCodTaxAmount > $baseTaxAmount) {
                $newMspCodBaseTaxAmount = abs($baseTaxAmount - $oldMspCodTaxAmount);
                $newMspCodTaxAmount = abs($taxAmount - $oldMspCodTaxAmount);
                $creditMemoBaseTaxAmount = $creditmemo->getBaseTaxAmount() - $newMspCodBaseTaxAmount;
                $creditMemoTaxAmount = $creditmemo->getTaxAmount() - $newMspCodTaxAmount;
                $creditMemoBaseGrandTotal = $creditmemo->getBaseGrandTotal() - $newMspCodBaseTaxAmount;
                $creditMemoGrandTotal = $creditmemo->getGrandTotal() - $newMspCodTaxAmount;
            } else {
                //Al ricalcolo dei valori della nota di credito, senza cambiare nulla sul cashondelivery, bisogna riaggiungere le tasse (da scoprire perchè, questo è un workaround)
                $creditMemoBaseTaxAmount = $creditmemo->getBaseTaxAmount() + $newMspCodBaseTaxAmount;
                $creditMemoTaxAmount = $creditmemo->getTaxAmount() + $newMspCodTaxAmount;
                $creditMemoBaseGrandTotal = $creditmemo->getBaseGrandTotal() + $newMspCodBaseTaxAmount;
                $creditMemoGrandTotal = $creditmemo->getGrandTotal() + $newMspCodTaxAmount;
            }

            $creditmemo->setMspCodAmount($this->priceCurrencyInterface->convert($baseMspCodAmount));
            $creditmemo->setBaseMspCodTaxAmount($newMspCodBaseTaxAmount);
            $creditmemo->setMspCodTaxAmount($newMspCodTaxAmount);
            $creditmemo->setBaseTaxAmount($creditMemoBaseTaxAmount);
            $creditmemo->setTaxAmount($creditMemoTaxAmount);
            $creditmemo->setBaseGrandTotal($creditMemoBaseGrandTotal);
            $creditmemo->setGrandTotal($creditMemoGrandTotal);
            $creditmemo->setHasChangedBaseMspCodAmount(false);
        }

        $creditMemoBaseGrandTotal = $creditmemo->getBaseGrandTotal();
        $creditMemoGrandTotal = $creditmemo->getGrandTotal();
        if($this->dataHelper->calculationCodInclTax()) {
            $creditMemoBaseGrandTotal += ($creditmemo->getBaseMspCodAmount() - $creditmemo->getBaseMspCodTaxAmount());
            $creditMemoGrandTotal += ($creditmemo->getMspCodAmount() - $creditmemo->getMspCodTaxAmount());
        } else {
            $creditMemoBaseGrandTotal += $creditmemo->getBaseMspCodAmount();
            $creditMemoGrandTotal += $creditmemo->getMspCodAmount();
        }

        if ($this->_canApplyTotal($order)) {
            $creditmemo->setBaseGrandTotal($creditMemoBaseGrandTotal);
            $creditmemo->setGrandTotal($creditMemoGrandTotal);
        }

        return $this;
    }
}
